package com.example.demo;


import org.apache.spark.sql.*;
import org.apache.spark.sql.streaming.StreamingQueryException;
import org.apache.spark.storage.StorageLevel;

import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.TimeoutException;

public class SparkSamples {
    public static void main(String[] args) throws StreamingQueryException, TimeoutException {
        SparkSession sparkSession = SparkSession
                .builder().master("local[*]").appName("Spark Streaming Example").getOrCreate();

        Dataset<Row> dataset = sparkSession.read().
                format("csv").option("header",true).
                load("src/main/resources/productData.csv");

        dataset.show();

        publishCsvRecordsToKafkaTopic(dataset);

    }

    private static void publishCsvRecordsToKafkaTopic(Dataset<Row> rowDataset) throws TimeoutException {
        if (null != rowDataset) {
            Dataset<Row> KafkaPublishJson = rowDataset
                    .withColumn("value", functions.to_json(functions.struct(functions.col("productId"),
                            functions.col("productName"), functions.col("productPrice"),
                            functions.col("deliveryStatus"), functions.col("date"))))
                    .alias("value").select("value")
                    .persist(StorageLevel.MEMORY_AND_DISK());
            KafkaPublishJson
                    .write().format("kafka").options(KafkaCsvConfig()).save();
        }
    }

    private static Map<String,String> KafkaCsvConfig(){
        Map<String, String> kafkaConfigMap = new HashMap<>();
        kafkaConfigMap.put("kafka.bootstrap.servers","127.0.0.1:9092");
        kafkaConfigMap.put("topic", "SparkOutput");
        return kafkaConfigMap;
    }
}
